"""
A special dataclass-like structure to handle tokens in xtsv format.
The WordFactory should be instantiated in the prepare_fields() method
of the internal app object of an xtsv module and stored as an attribute
value of the internal app.
The get_word() method should be called in the process_sentence() method
of the internal app, and each input token in the xtsv sentence stream
should be passed to it.
"""

from types import SimpleNamespace


class WordFactory:
    """
    Factory class that produces Word objects according
    to the specified settings.
    Initialize with the value of the field_names argument of the
    prepare_fields() method and with either the value of
    self.target_fields, which is a list of output fields added by
    the current xtsv module, or with a dict that contains
    'field: initial value' pairs for each target field.
    If only the target_fields list is passed, the values of
    these fields are all initialized to target_value_default.
    """

    def __init__(self, xtsv_field_names, /,
                 target_fields=None, *,
                 target_value_default='',
                 target_field_inits=None):
        if target_field_inits is not None:
            self.target_fields = target_field_inits
        else:
            self.target_fields = {field: target_value_default
                                         for field in target_fields}

        self.input_fields = [field
                             for field in xtsv_field_names
                             if (isinstance(field, str)
                                 and field not in self.target_fields)]

    def __repr__(self):
        return (f"WordFactory(input_fields={self.input_fields!r}, "
                + f"target_fields={self.target_fields!r})")

    def get_word(self, values):
        """
        Return a Word object the input fields of which are
        initialised to the specified values.
        The target fields are initialised according to the
        settings of the factory object.
        Raise a ValueError if the number of input fields and
        the length of the provided iterable of values do not match.
        """

        try:
            input_values = dict(zip(self.input_fields, values, strict=True))
        except ValueError as exc:
            raise ValueError(
                f"{len(self.input_fields)} values expected, "
                + f"{len(values)} provided:\n"
                + f"Features: {self.input_fields}\n"
                + f"Values: {values}") from exc
        return Word(input_values | self.target_fields)


class Word(SimpleNamespace):
    """
    Convenience class to access (get and set) the fields
    (i.e. features) of an xtsv token as attributes or
    alternatively as dictionary keys.
    Yields the values of each field in order when used
    as an iterable.
    Additional attributes can be freely added to a Word
    object. They are ignored in the string representation,
    the iterable, and the dict functions, so they will not
    break the integrity of the xtsv output.
    """

    def __init__(self, value_dict):
        super().__init__(**value_dict)
        self._word_attrs = list(value_dict.keys())

    def __repr__(self):
        return "Word({})".format(', '.join(f"{k}={v!r}" for k, v
                                           in self.__dict__.items()
                                           if k != '_word_attrs'))

    def __iter__(self):
        return iter(self.__dict__[key] for key in self._word_attrs)

    def __getitem__(self, key):
        return self.__dict__[key]

    def __len__(self):
        return len(self._word_attrs)

    def keys(self):
        '''Get names of features'''
        return {key: self.__dict__[key] for key in self._word_attrs}.keys()

    def values(self):
        '''Get value of each feature'''
        return {key: self.__dict__[key] for key in self._word_attrs}.values()

    def items(self):
        '''Get feature-value pairs'''
        return {key: self.__dict__[key] for key in self._word_attrs}.items()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xtsv_word']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'xtsv-word',
    'version': '1.0.1',
    'description': 'A dataclass-like structure to make the handling of xtsv token attributes comfortable and transparent',
    'long_description': "# xtsv-word\n\n**This module provides a special dataclass-like structure to handle tokens in [xtsv](https://github.com/nytud/xtsv) format. It is meant to be used within `xtsv` modules to make the processing of token attributes (`xtsv` fields) more comfortable and transparent.**\n\nIt allows for each token to be represented by a `Word` object which is initialised simply by passing the token in its `xtsv` representation (i.e. a list of strings, one for each field in the input stream) to a `WordFactory` object. This factory object keeps track of the input and target fields of the `xtsv` module, and assigns the items of the list representing the token in `xtsv` to the respective `Word` object attributes which are identified by the name of the corresponding field (i.e. the `xtsv` column header).\n\nBoth the input and the target fields can be accessed as attributes of a `Word` object, i.e. they can be both retrieved and modified. (The usual use case is to only read input field attributes and to specify target field attributes. However, the `Word` object does not prevent a user from modifying input field values. This is discouraged but not ruled out by `xtsv`.) When the `xtsv` module is done processing a token, the `Word` object is simply converted into a list which contains the original input fields followed by the target fields, as expected by `xtsv`.\n\nDisclaimer: This is not an official extension of the [xtsv](https://github.com/nytud/xtsv) module.\n\n## Suggested usage\n\nInstall `xtsv-word` from `pip`:\n```\npython3 -m pip install xtsv-word\n```\n\nor build locally:\n```\nmake\n```\n\nFor example, assuming that the internal app object defined in the `xtsv` module `myXtsvModule` is called `InternalApp`, the input stream contains the fields `['form', 'wsafter']` and `myXtsvModule` has a single target field: `['syllables']`:\n\n1. **Create `WordFactory` object:**\n\n```\n# myXtsvModule.py\n\nfrom xtsv_word import WordFactory\n\nclass InternalApp:\n\t...\n\tdef prepare_fields(self, field_names):\n\t\tself.wf = WordFactory(field_names, self.target_fields)\n\t\t# self.target_fields is normally set in InternalApp.__init__()\n```\n\n2. **Use `Word` object:**\n\n```\nclass InternalApp:\n\t...\n\tdef process_sentence(self, sen, field_values):\n\t\treturn_sen = []\n\t\tfor tok in sen:\n\t\t\t# Get Word object from factory\n\t\t\tword = self.wf.get_word(tok)\n\n\t\t\t# process token by getting and setting its attributes, e.g.:\n\t\t\tword.syllables = split_syllables(word.form)\n\t\t\t...\n\t\t\t# alternatively access attributes as dict keys:\n\t\t\tword['syllables'] = '-'.join(word['syllables'])\n\n\t\t\t# convert Word object to list of fields for xtsv output stream\n\t\t\treturn_sen.append(list(word))\n\n\t\t...\n\t\treturn return_sen\n```\n\nSee docstrings for further details.\n",
    'author': 'Pethő Gergely',
    'author_email': 'pagstudium@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ril-lexknowrep/xtsv_word',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
